@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2010 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -44,7 +44,7 @@
 #define UWM_SYSTRAY (WM_USER + 1) /* sent to us by the taskbar */
 #define LOG_LINES 250
 
-/* Prototypes */
+/* prototypes */
 static void parse_cmdline(LPSTR);
 #ifndef _WIN32_WCE
 static int set_cwd(void);
@@ -71,7 +71,7 @@ static void WINAPI service_main(DWORD, LPTSTR *);
 static void WINAPI control_handler(DWORD);
 #endif /* !defined(_WIN32_WCE) */
 
-/* Global variables */
+/* global variables */
 static struct LIST {
   struct LIST *next;
   int len;
@@ -95,7 +95,6 @@ TCHAR win32_name[STRLEN];
 static SERVICE_STATUS serviceStatus;
 static SERVICE_STATUS_HANDLE serviceStatusHandle=0;
 #endif
-static HANDLE stopServiceEvent=0;
 
 static int visible=0, error_mode=0;
 static jmp_buf jump_buf;
@@ -139,8 +138,8 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
         return 1;
 #endif
 
-    /* setup the windo caption before reading the configuration file
-     * options.win32_service is not available here and may not be used */
+    /* setup the initial window caption before reading the configuration file
+     * global_options.win32_service may not be used here */
 #ifdef _WIN32_WCE
     _tcscpy(win32_name, TEXT("stunnel ") TEXT(VERSION)
         TEXT(" on Windows CE (not configured)"));
@@ -160,7 +159,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
             TEXT(" on Windows CE"));
 #else
         _snprintf(win32_name, STRLEN, "stunnel %s on Win32 (%s)",
-            VERSION, options.win32_service); /* update the information */
+            VERSION, global_options.win32_service); /* update the information */
         if(!cmdline.service) {
             if(cmdline.install)
                 return service_install(command_line);
@@ -172,6 +171,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                 return service_stop();
         }
 #endif
+    } else {
     }
 
     /* CATCH */
@@ -187,7 +187,7 @@ static void parse_cmdline(LPSTR command_line) {
     char line[STRLEN], *c, *opt;
 
     safecopy(line, command_line);
-    memset(&cmdline, 0, sizeof(cmdline));
+    memset(&cmdline, 0, sizeof cmdline);
 
     c=line;
     while(*c && (*c=='/' || *c=='-')) {
@@ -257,7 +257,7 @@ static int initialize_winsock() {
             return 0; /* IPv6 detected -> OK */
         FreeLibrary(handle);
     }
-    handle=LoadLibrary("wship6.dll"); /* Experimental IPv6 for Windows 2000 */
+    handle=LoadLibrary("wship6.dll"); /* experimental IPv6 for Windows 2000 */
     if(handle) {
         s_getaddrinfo=(GETADDRINFO)GetProcAddress(handle, "getaddrinfo");
         s_freeaddrinfo=(FREEADDRINFO)GetProcAddress(handle, "freeaddrinfo");
@@ -285,7 +285,7 @@ static int win_main(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
     /* register the class */
 #ifndef _WIN32_WCE
-    wc.cbSize=sizeof(WNDCLASSEX);
+    wc.cbSize=sizeof wc;
 #endif
     wc.style=CS_VREDRAW|CS_HREDRAW;
     wc.lpfnWndProc=wndProc;
@@ -306,7 +306,7 @@ static int win_main(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 #endif
 
     /* create main window */
-    if(options.option.taskbar) { /* save menu resources */
+    if(global_options.option.taskbar) { /* save menu resources */
         htraymenu=LoadMenu(ghInst, MAKEINTRESOURCE(IDM_TRAYMENU));
         hpopup=GetSubMenu(htraymenu, 0);
     }
@@ -320,14 +320,16 @@ static int win_main(HINSTANCE hInstance, HINSTANCE hPrevInstance,
     hwnd=CreateWindow(classname, win32_name, WS_TILEDWINDOW,
         CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
         NULL, hmainmenu, hInstance, NULL);
-#endif
 
     if(cmdline.service) /* do not allow to save file in the service mode */
         EnableMenuItem(hmainmenu, IDM_SAVEAS, MF_GRAYED);
+#endif
 
-    if(error_mode) /* log window is hidden by default */
+    if(error_mode) { /* log window is hidden by default */
         set_visible(1);
-    else /* create the main thread */
+        EnableMenuItem(hmainmenu, IDM_RELOAD, MF_GRAYED);
+        EnableMenuItem(htraymenu, IDM_RELOAD, MF_GRAYED);
+    } else /* create the main thread */
         _beginthread(ThreadFunc, 0, NULL);
 
     while(GetMessage(&msg, NULL, 0, 0)) {
@@ -341,8 +343,8 @@ static int win_main(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 static void update_taskbar(void) { /* create the taskbar icon */
     NOTIFYICONDATA nid;
 
-    ZeroMemory(&nid, sizeof(nid));
-    nid.cbSize=sizeof(NOTIFYICONDATA); /* size */
+    ZeroMemory(&nid, sizeof nid);
+    nid.cbSize=sizeof nid; /* size */
     nid.hWnd=hwnd; /* window to receive notifications */
     nid.uID=1;     /* application-defined ID for icon */
     if(error_mode)
@@ -362,10 +364,14 @@ static void update_taskbar(void) { /* create the taskbar icon */
 }
 
 static void ThreadFunc(void *arg) {
-    if(!setjmp(jump_buf))
+    if(!setjmp(jump_buf)) {
         main_execute();
-    else
-        set_visible(1); /* could be unsafe to call it from another thread */
+    } else {
+        /* FIXME: could be unsafe to call it from another thread */
+        set_visible(1);
+        EnableMenuItem(hmainmenu, IDM_RELOAD, MF_GRAYED);
+        EnableMenuItem(htraymenu, IDM_RELOAD, MF_GRAYED);
+    }
     _endthread();
 }
 
@@ -381,7 +387,7 @@ static LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM l
 #endif
     switch(message) {
     case WM_CREATE:
-        if(options.option.taskbar) /* taskbar update enabled? */
+        if(global_options.option.taskbar) /* taskbar update enabled? */
             SetTimer(hwnd, 0x29a, 1000, NULL); /* 1-second timer */
 
 #ifdef _WIN32_WCE
@@ -443,8 +449,8 @@ static LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM l
 #endif
         if(htraymenu)
             DestroyMenu(htraymenu);
-        ZeroMemory(&nid, sizeof(nid));
-        nid.cbSize=sizeof(NOTIFYICONDATA);
+        ZeroMemory(&nid, sizeof nid);
+        nid.cbSize=sizeof nid;
         nid.hWnd=hwnd;
         nid.uID=1;
         nid.uFlags=NIF_TIP; /* not really sure what to put here, but it works */
@@ -470,15 +476,17 @@ static LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM l
         case IDM_SAVEAS:
             save_file(hwnd);
             break;
-        case IDM_SETUP:
-            MessageBox(hwnd, TEXT("Function not implemented"),
-                win32_name, MB_ICONERROR);
+        case IDM_RELOAD:
+            log_close();
+            parse_conf(NULL, CONF_RELOAD);
+            log_open();
+            bind_ports();
             break;
         }
         return TRUE;
 
     case UWM_SYSTRAY: /* a taskbar event */
-        switch (lParam) {
+        switch(lParam) {
 #ifdef _WIN32_WCE
         case WM_LBUTTONDOWN: /* no right mouse button on Windows CE */
             GetWindowRect(GetDesktopWindow(), &rect); /* no cursor position */
@@ -523,24 +531,29 @@ static LRESULT CALLBACK pass_proc(HWND hDlg, UINT message,
         WPARAM wParam, LPARAM lParam) {
     TCHAR titlebar[STRLEN];
     WORD cchPassword;
+    LPTSTR keyFileName;
+    TCHAR sPassword[PEM_BUFSIZE];
+    char* cPassword;
 
-    switch (message) {
+    switch(message) {
     case WM_INITDIALOG:
-        /* Set the default push button to "Cancel." */
+        /* set the default push button to "Cancel." */
         SendMessage(hDlg, DM_SETDEFID, (WPARAM) IDCANCEL, (LPARAM) 0);
 
+        keyFileName = str2tstr(ui_data->section->key);
         _sntprintf(titlebar, STRLEN, TEXT("Private key: %s"),
-            ui_data->section->key);
+            keyFileName);
+        free(keyFileName);    
         SetWindowText(hDlg, titlebar);
         return TRUE;
 
     case WM_COMMAND:
-        /* Set the default push button to "OK" when the user enters text. */
+        /* set the default push button to "OK" when the user enters text */
         if(HIWORD (wParam) == EN_CHANGE && LOWORD(wParam) == IDE_PASSEDIT)
             SendMessage(hDlg, DM_SETDEFID, (WPARAM) IDOK, (LPARAM) 0);
         switch(wParam) {
         case IDOK:
-            /* Get number of characters. */
+            /* get number of characters */
             cchPassword=(WORD)SendDlgItemMessage(hDlg,
                 IDE_PASSEDIT, EM_LINELENGTH, (WPARAM) 0, (LPARAM) 0);
             if(!cchPassword || cchPassword>=PEM_BUFSIZE) {
@@ -548,14 +561,19 @@ static LRESULT CALLBACK pass_proc(HWND hDlg, UINT message,
                 return FALSE;
             }
 
-            /* Put the number of characters into first word of buffer. */
-            *((LPWORD)ui_data->pass) = cchPassword;
+            /* put the number of characters into first word of buffer */
+            *((LPWORD) sPassword)=cchPassword;
 
-            /* Get the characters. */
+            /* get the characters */
             SendDlgItemMessage(hDlg, IDE_PASSEDIT, EM_GETLINE,
-                (WPARAM) 0, /* line 0 */ (LPARAM) ui_data->pass);
+                (WPARAM) 0, /* line 0 */ (LPARAM)sPassword);
+            sPassword[cchPassword]='\0'; /* null-terminate the string */
+            
+            /* convert input password to ANSI string (as ui_data->pass) */
+            cPassword = tstr2str(sPassword);
+            strcpy(ui_data->pass, cPassword);
+            free(cPassword);
 
-            ui_data->pass[cchPassword] = 0; /* Null-terminate the string. */
             EndDialog(hDlg, TRUE);
             return TRUE;
 
@@ -601,10 +619,10 @@ static void save_file(HWND hwnd) {
     if(cmdline.service) /* do not allow to save file in the service mode */
         return;
 
-    ZeroMemory(&ofn, sizeof(ofn));
+    ZeroMemory(&ofn, sizeof ofn);
     szFileName[0]='\0';
 
-    ofn.lStructSize=sizeof(ofn);
+    ofn.lStructSize=sizeof ofn;
     ofn.hwndOwner=hwnd;
     ofn.lpstrFilter=TEXT("Log Files (*.log)\0*.log\0All Files (*.*)\0*.*\0\0");
     ofn.lpstrFile=szFileName;
@@ -635,7 +653,7 @@ static void save_file(HWND hwnd) {
     CloseHandle(hFile);
 }
 
-void win_log(LPSTR line) { /* Also used in log.c */
+void win_log(LPSTR line) { /* also used in log.c */
     struct LIST *curr;
     int len;
     static int log_len=0;
@@ -739,11 +757,11 @@ static void error_box(const LPTSTR text) {
 
 static int service_initialize(void) {
     SERVICE_TABLE_ENTRY serviceTable[]={
-        {options.win32_service, service_main},
+        {global_options.win32_service, service_main},
         {0, 0}
     };
 
-    options.option.taskbar=0; /* disable taskbar for security */
+    global_options.option.taskbar=0; /* disable taskbar for security */
     if(!StartServiceCtrlDispatcher(serviceTable)) {
         error_box(TEXT("StartServiceCtrlDispatcher"));
         return 1;
@@ -766,9 +784,13 @@ static int service_install(LPSTR command_line) {
     safeconcat(service_path, "\" -service ");
     safeconcat(service_path, command_line);
     service=CreateService(scm,
-        options.win32_service, options.win32_service, SERVICE_ALL_ACCESS,
+        global_options.win32_service,
+        global_options.win32_service,
+        SERVICE_ALL_ACCESS,
         SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS,
-        SERVICE_AUTO_START, SERVICE_ERROR_NORMAL, service_path,
+        SERVICE_AUTO_START,
+        SERVICE_ERROR_NORMAL,
+        service_path,
         NULL, NULL, NULL, NULL, NULL);
     if(!service) {
         error_box(TEXT("CreateService"));
@@ -792,7 +814,7 @@ static int service_uninstall(void) {
         error_box(TEXT("OpenSCManager"));
         return 1;
     }
-    service=OpenService(scm, options.win32_service,
+    service=OpenService(scm, global_options.win32_service,
         SERVICE_QUERY_STATUS | DELETE);
     if(!service) {
         if(!cmdline.quiet)
@@ -836,7 +858,7 @@ static int service_start(void) {
         error_box(TEXT("OpenSCManager"));
         return 1;
     }
-    service=OpenService(scm, options.win32_service,
+    service=OpenService(scm, global_options.win32_service,
         SERVICE_QUERY_STATUS | SERVICE_START);
     if(!service) {
         error_box(TEXT("OpenService"));
@@ -850,7 +872,7 @@ static int service_start(void) {
         return 1;
     }
     do {
-        sleep(1);
+        Sleep(1000);
         if(!QueryServiceStatus(service, &serviceStatus)) {
             error_box(TEXT("QueryServiceStatus"));
             CloseServiceHandle(service);
@@ -882,7 +904,7 @@ static int service_stop(void) {
         error_box(TEXT("OpenSCManager"));
         return 1;
     }
-    service=OpenService(scm, options.win32_service,
+    service=OpenService(scm, global_options.win32_service,
         SERVICE_QUERY_STATUS | SERVICE_STOP);
     if(!service) {
         if(!cmdline.quiet)
@@ -911,7 +933,7 @@ static int service_stop(void) {
         return 1;
     }
     do {
-        sleep(1);
+        Sleep(1000);
         if(!QueryServiceStatus(service, &serviceStatus)) {
             error_box(TEXT("QueryServiceStatus"));
             CloseServiceHandle(service);
@@ -938,16 +960,14 @@ static void WINAPI service_main(DWORD argc, LPTSTR* argv) {
     serviceStatus.dwWaitHint=0;
 
     serviceStatusHandle=
-        RegisterServiceCtrlHandler(options.win32_service, control_handler);
+        RegisterServiceCtrlHandler(global_options.win32_service,
+            control_handler);
 
     if(serviceStatusHandle) {
         /* service is starting */
         serviceStatus.dwCurrentState=SERVICE_START_PENDING;
         SetServiceStatus(serviceStatusHandle, &serviceStatus);
 
-        /* do initialisation here */
-        stopServiceEvent=CreateEvent(0, FALSE, FALSE, 0);
-
         /* running */
         serviceStatus.dwControlsAccepted|=
             (SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN);
@@ -960,10 +980,6 @@ static void WINAPI service_main(DWORD argc, LPTSTR* argv) {
         serviceStatus.dwCurrentState=SERVICE_STOP_PENDING;
         SetServiceStatus(serviceStatusHandle, &serviceStatus);
 
-        /* do cleanup here */
-        CloseHandle(stopServiceEvent);
-        stopServiceEvent=0;
-
         /* service is now stopped */
         serviceStatus.dwControlsAccepted&=
             ~(SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN);
@@ -973,7 +989,7 @@ static void WINAPI service_main(DWORD argc, LPTSTR* argv) {
 }
 
 static void WINAPI control_handler(DWORD controlCode) {
-    switch (controlCode) {
+    switch(controlCode) {
     case SERVICE_CONTROL_INTERROGATE:
         break;
 
@@ -982,7 +998,6 @@ static void WINAPI control_handler(DWORD controlCode) {
         serviceStatus.dwCurrentState=SERVICE_STOP_PENDING;
         SetServiceStatus(serviceStatusHandle, &serviceStatus);
         PostMessage(hwnd, WM_COMMAND, IDM_EXIT, 0);
-        SetEvent(stopServiceEvent);
         return;
 
     case SERVICE_CONTROL_PAUSE:
@@ -1003,4 +1018,4 @@ static void WINAPI control_handler(DWORD controlCode) {
 
 #endif /* !defined(_WIN32_WCE) */
 
-/* End of gui.c */
+/* end of gui.c */
