@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2010 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
@@ -33,12 +40,14 @@
 
 /* \n is not a character expected in the string */
 #define LINE "%[^\n]"
-
 #define isprefix(a, b) (strncasecmp((a), (b), strlen(b))==0)
+#define s_min(a, b) ((a)>(b)?(b):(a))
 
 /* protocol-specific function prototypes */
 static void cifs_client(CLI *);
 static void cifs_server(CLI *);
+static void pgsql_client(CLI *);
+static void pgsql_server(CLI *);
 static void smtp_client(CLI *);
 static void smtp_server(CLI *);
 static void pop3_client(CLI *);
@@ -54,7 +63,7 @@ static char *base64(int, char *, int);
 
 void negotiate(CLI *c) {
     if(!c->opt->protocol)
-        return; /* No protocol negotiations */
+        return; /* no protocol negotiations */
 
     s_log(LOG_NOTICE, "Negotiations for %s (%s side) started", c->opt->protocol,
         c->opt->option.client ? "client" : "server");
@@ -72,6 +81,8 @@ void negotiate(CLI *c) {
             nntp_client(c);
         else if(!strcmp(c->opt->protocol, "connect"))
             connect_client(c);
+        else if(!strcmp(c->opt->protocol, "pgsql"))
+            pgsql_client(c);
         else {
             s_log(LOG_ERR, "Protocol %s not supported in client mode",
                 c->opt->protocol);
@@ -86,6 +97,8 @@ void negotiate(CLI *c) {
             pop3_server(c);
         else if(!strcmp(c->opt->protocol, "imap"))
             imap_server(c);
+        else if(!strcmp(c->opt->protocol, "pgsql"))
+            pgsql_server(c);
         else {
             s_log(LOG_ERR, "Protocol %s not supported in server mode",
                 c->opt->protocol);
@@ -124,7 +137,7 @@ static void cifs_server(CLI *c) {
     read_blocking(c, c->local_rfd.fd, buffer, 4) ;/* NetBIOS header */
     len=buffer[3];
     len|=(u16)(buffer[2]) << 8;
-    if(len>sizeof(buffer)-4) {
+    if(len>sizeof buffer-4) {
         s_log(LOG_ERR, "Received block too long");
         longjmp(c->err, 1);
     }
@@ -137,28 +150,56 @@ static void cifs_server(CLI *c) {
     write_blocking(c, c->local_wfd.fd, response_use_ssl, 5);
 }
 
+/* http://www.postgresql.org/docs/8.3/static/protocol-flow.html#AEN73013 */
+u8 ssl_request[8]={0, 0, 0, 8, 0x04, 0xd2, 0x16, 0x2f};
+
+static void pgsql_client(CLI *c) {
+    u8 buffer[1];
+
+    write_blocking(c, c->remote_fd.fd, ssl_request, sizeof ssl_request);
+    read_blocking(c, c->remote_fd.fd, buffer, 1);
+    /* S - accepted, N - rejected, non-SSL preferred */
+    if(buffer[0]!='S') {
+        s_log(LOG_ERR, "PostgreSQL server rejected SSL");
+        longjmp(c->err, 1);
+    }
+}
+
+static void pgsql_server(CLI *c) {
+    u8 buffer[8], ssl_ok[1]={'S'};
+
+    memset(buffer, 0, sizeof buffer);
+    read_blocking(c, c->local_rfd.fd, buffer, sizeof buffer);
+    if(memcmp(buffer, ssl_request, sizeof ssl_request)) {
+        s_log(LOG_ERR, "PostgreSQL client did not request SSL, rejecting");
+        /* no way to send error on startup, so just drop the client */
+        longjmp(c->err, 1);
+    }
+    write_blocking(c, c->local_wfd.fd, ssl_ok, sizeof ssl_ok);
+}
+
 static void smtp_client(CLI *c) {
     char line[STRLEN];
 
-    do { /* Copy multiline greeting */
+    do { /* copy multiline greeting */
         fdgetline(c, c->remote_fd.fd, line);
         fdputline(c, c->local_wfd.fd, line);
     } while(isprefix(line, "220-"));
 
     fdputline(c, c->remote_fd.fd, "EHLO localhost");
-    do { /* Skip multiline reply */
+    do { /* skip multiline reply */
         fdgetline(c, c->remote_fd.fd, line);
     } while(isprefix(line, "250-"));
-    if(!isprefix(line, "250 ")) { /* Error */
+    if(!isprefix(line, "250 ")) { /* error */
         s_log(LOG_ERR, "Remote server is not RFC 1425 compliant");
         longjmp(c->err, 1);
     }
 
     fdputline(c, c->remote_fd.fd, "STARTTLS");
-    do { /* Skip multiline reply */
+    do { /* skip multiline reply */
         fdgetline(c, c->remote_fd.fd, line);
     } while(isprefix(line, "220-"));
-    if(!isprefix(line, "220 ")) { /* Error */
+    if(!isprefix(line, "220 ")) { /* error */
         s_log(LOG_ERR, "Remote server is not RFC 2487 compliant");
         longjmp(c->err, 1);
     }
@@ -167,7 +208,7 @@ static void smtp_client(CLI *c) {
 static void smtp_server(CLI *c) {
     char line[STRLEN];
 
-    s_poll_zero(&c->fds);
+    s_poll_init(&c->fds);
     s_poll_add(&c->fds, c->local_rfd.fd, 1, 0);
     switch(s_poll_wait(&c->fds, 0, 200)) { /* wait up to 200ms */
     case 0: /* fd not ready to read */
@@ -175,7 +216,7 @@ static void smtp_server(CLI *c) {
         break;
     case 1: /* fd ready to read */
         s_log(LOG_DEBUG, "RFC 2487 not detected");
-        return; /* Return if RFC 2487 is not used */
+        return; /* return if RFC 2487 is not used */
     default: /* -1 */
         sockerror("RFC2487 (s_poll_wait)");
         longjmp(c->err, 1);
@@ -225,7 +266,7 @@ static void pop3_server(CLI *c) {
     fdgetline(c, c->remote_fd.fd, line);
     fdprintf(c, c->local_wfd.fd, "%s + stunnel", line);
     fdgetline(c, c->local_rfd.fd, line);
-    if(isprefix(line, "CAPA")) { /* Client wants RFC 2449 extensions */
+    if(isprefix(line, "CAPA")) { /* client wants RFC 2449 extensions */
         fdputline(c, c->local_wfd.fd, "-ERR Stunnel does not support capabilities");
         fdgetline(c, c->local_rfd.fd, line);
     }
@@ -258,7 +299,7 @@ static void imap_client(CLI *c) {
 static void imap_server(CLI *c) {
     char line[STRLEN], id[STRLEN], *tail, *capa;
  
-    s_poll_zero(&c->fds);
+    s_poll_init(&c->fds);
     s_poll_add(&c->fds, c->local_rfd.fd, 1, 0);
     switch(s_poll_wait(&c->fds, 0, 200)) {
     case 0: /* fd not ready to read */
@@ -266,7 +307,7 @@ static void imap_server(CLI *c) {
         break;
     case 1: /* fd ready to read */
         s_log(LOG_DEBUG, "RFC 2595 not detected");
-        return; /* Return if RFC 2595 is not used */
+        return; /* return if RFC 2595 is not used */
     default: /* -1 */
         sockerror("RFC2595 (s_poll_wait)");
         longjmp(c->err, 1);
@@ -357,6 +398,8 @@ static void nntp_client(CLI *c) {
 
 static void connect_client(CLI *c) {
     char line[STRLEN], ntlm2[STRLEN], *encoded;
+    long content_length;
+    char buf[BUFSIZ];
 
     if(!c->opt->protocol_host) {
         s_log(LOG_ERR, "protocolHost not specified");
@@ -369,6 +412,7 @@ static void connect_client(CLI *c) {
         if(!strcasecmp(c->opt->protocol_authentication, "NTLM")) {
 
             /* send Proxy-Authorization (phase 1) */
+            fdprintf(c, c->remote_fd.fd, "Proxy-Connection: keep-alive");
             fdprintf(c, c->remote_fd.fd, "Proxy-Authorization: NTLM %s",
                 ntlm1());
             fdputline(c, c->remote_fd.fd, ""); /* empty line */
@@ -383,12 +427,22 @@ static void connect_client(CLI *c) {
                 longjmp(c->err, 1);
             }
             *ntlm2='\0';
+            content_length=0; /* no HTTP content */
             do { /* read all headers */
                 fdgetline(c, c->remote_fd.fd, line);
                 if(isprefix(line, "Proxy-Authenticate: NTLM "))
                     safecopy(ntlm2, line+25);
+                else if(isprefix(line, "Content-Length: "))
+                    content_length=atol(line+16);
             } while(*line);
 
+            /* read and ignore HTTP content (if any) */
+            while(content_length) {
+                read_blocking(c, c->remote_fd.fd, buf,
+                    s_min(content_length, BUFSIZ));
+                content_length-=s_min(content_length, BUFSIZ);
+            }
+
             /* send Proxy-Authorization (phase 3) */
             fdprintf(c, c->remote_fd.fd, "CONNECT %s HTTP/1.1",
                 c->opt->protocol_host);
@@ -433,26 +487,24 @@ static void connect_client(CLI *c) {
 static char *ntlm1() {
     char phase1[16];
 
-    memset(phase1, 0, sizeof(phase1));
+    memset(phase1, 0, sizeof phase1);
     strcpy(phase1, "NTLMSSP");
     phase1[8]=1; /* type: 1 */
     phase1[12]=2; /* flag: negotiate OEM */
     phase1[13]=2; /* flag: negotiate NTLM */
-    return base64(1, phase1, sizeof(phase1)); /* encode */
+    return base64(1, phase1, sizeof phase1); /* encode */
 }
 
-#define s_min(a, b) ((a)>(b)?(b):(a))
-
 static char *ntlm3(char *username, char *password, char *phase2) {
+    MD4_CTX md4;
     char *decoded; /* decoded reply from proxy */
     char phase3[146];
-    char md4_hash[21];
+    unsigned char md4_hash[21];
     int userlen=strlen(username);
-    int phase3len=s_min(88+userlen, sizeof(phase3));
-    MD4_CTX md4;
+    int phase3len=s_min(88+userlen, sizeof phase3);
 
     /* setup phase3 structure */
-    memset(phase3, 0, sizeof(phase3));
+    memset(phase3, 0, sizeof phase3);
     strcpy(phase3, "NTLMSSP");
     phase3[8]=3;            /* type: 3 */
     phase3[16]=phase3len;   /* LM-resp off */
@@ -479,12 +531,15 @@ static char *ntlm3(char *username, char *password, char *phase2) {
 
     /* decode challenge and calculate response */
     decoded=base64(0, phase2, strlen(phase2)); /* decode */
-    crypt_DES(phase3+64, decoded+24, md4_hash);
-    crypt_DES(phase3+72, decoded+24, md4_hash+7);
-    crypt_DES(phase3+80, decoded+24, md4_hash+14);
+    crypt_DES((unsigned char *)phase3+64,
+        (unsigned char *)decoded+24, md4_hash);
+    crypt_DES((unsigned char *)phase3+72,
+        (unsigned char *)decoded+24, md4_hash+7);
+    crypt_DES((unsigned char *)phase3+80,
+        (unsigned char *)decoded+24, md4_hash+14);
     free(decoded);
 
-    strncpy(phase3+88, username, sizeof(phase3)-88);
+    strncpy(phase3+88, username, sizeof phase3-88);
 
     return base64(1, phase3, phase3len); /* encode */
 }
@@ -520,7 +575,7 @@ static char *base64(int encode, char *in, int len) {
     if(encode)
         bio=BIO_push(b64, bio);
     BIO_write(bio, in, len);
-    BIO_flush(bio);
+    (void)BIO_flush(bio); /* ignore the error if any */
     if(encode) {
         bio=BIO_pop(bio);
         BIO_free(b64);
@@ -528,14 +583,16 @@ static char *base64(int encode, char *in, int len) {
         bio=BIO_push(b64, bio);
     }
     len=BIO_pending(bio);
-    out=calloc(len+1, 1);
+    /* 32 bytes as a safety precaution for passing decoded data to crypt_DES */
+    /* len+1 to get null-terminated string on encode */
+    out=calloc(len<32?32:len+1, 1);
     if(!out) {
-        log_raw("Fatal memory allocation error");
-        exit(2);
+        s_log(LOG_ERR, "Fatal memory allocation error");
+        die(2);
     }
     BIO_read(bio, out, len);
     BIO_free_all(bio);
     return out;
 }
 
-/* End of protocol.c */
+/* end of protocol.c */
