@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2010 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -38,7 +38,7 @@
 #include "common.h"
 #include "prototypes.h"
 
-/**************************************** Resolver functions */
+/**************************************** resolver functions */
 
 #ifndef HAVE_GETADDRINFO
 
@@ -48,6 +48,9 @@
 #ifndef EAI_NONAME
 #define EAI_NONAME 2
 #endif
+#ifndef EAI_SERVICE
+#define EAI_SERVICE 8
+#endif
 
 #if defined(USE_WIN32) && defined(USE_IPv6)
 /* rename some locally shadowed declarations */
@@ -113,7 +116,7 @@ int hostport2addrlist(SOCKADDR_LIST *addr_list,
 
     addr_list->cur=0; /* initialize round-robin counter */
 
-    memset(&hints, 0, sizeof(hints));
+    memset(&hints, 0, sizeof hints);
 #if defined(USE_IPv6) || defined(USE_WIN32)
     hints.ai_family=PF_UNSPEC;
 #else
@@ -131,7 +134,7 @@ int hostport2addrlist(SOCKADDR_LIST *addr_list,
                 hostname, s_gai_strerror(err));
         if(res)
             freeaddrinfo(res);
-        return 0; /* Error */
+        return 0; /* error */
     }
     /* copy the list of addresses */
     cur=res;
@@ -332,7 +335,7 @@ static void freeaddrinfo(struct addrinfo *current) {
 
 #endif /* !defined HAVE_GETADDRINFO */
 
-/* Due to a problem with Mingw32 I decided to define my own gai_strerror() */
+/* due to a problem with Mingw32 I decided to define my own gai_strerror() */
 static const char *s_gai_strerror(int err) {
     switch(err) {
 #ifdef EAI_BADFLAGS
@@ -412,22 +415,4 @@ static int getnameinfo(const struct sockaddr *sa, int salen,
 }
 #endif
 
-/**************************************** Some optional debugging stuff */
-
-#if 0
-void debug_sockaddr_union(SOCKADDR_UNION *addr) {
-    int i;
-    char t[3*sizeof(SOCKADDR_UNION)];
-    const char hex[16]="0123456789abcdef";
-
-    for(i=0; i<sizeof(SOCKADDR_UNION); i++) {
-        t[3*i]=hex[((u8 *)addr)[i]/16];
-        t[3*i+1]=hex[((u8 *)addr)[i]%16];
-        t[3*i+2]=' ';
-    }
-    t[3*sizeof(SOCKADDR_UNION)-1]='\0';
-    s_log(LOG_DEBUG, "%s\n", t);
-}
-#endif
-
-/* End of resolver.c */
+/* end of resolver.c */
