@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2010 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -38,16 +38,16 @@
 #include "common.h"
 #include "prototypes.h"
 
-    /* Global OpenSSL initalization: compression, engine, entropy */
-static void init_compression(void);
+    /* global OpenSSL initalization: compression, engine, entropy */
+static int init_compression(void);
 static int init_prng(void);
 static int prng_seeded(int);
 static int add_rand_file(char *);
 #ifdef HAVE_OSSL_ENGINE_H
-static void init_engine(void);
+static char *init_engine(void);
 #endif
 
-int cli_index; /* to keep CLI structure for verify callback */
+int cli_index, opt_index; /* to keep structure for callbacks */
 
 void ssl_init(void) { /* init SSL before parsing configuration file */
 #ifdef HAVE_OPENSSL
@@ -56,24 +56,36 @@ void ssl_init(void) { /* init SSL before parsing configuration file */
     SSLeay_add_ssl_algorithms();
     SSL_load_error_strings();
     cli_index=SSL_get_ex_new_index(0, "cli index", NULL, NULL, NULL);
+    opt_index=SSL_CTX_get_ex_new_index(0, "opt index", NULL, NULL, NULL);
 #ifdef HAVE_OSSL_ENGINE_H
     ENGINE_load_builtin_engines();
 #endif
 }
 
-void ssl_configure(void) { /* configure global SSL settings */
-    if(options.compression!=COMP_NONE)
-        init_compression();
+int ssl_configure(void) { /* configure global SSL settings */
+#ifdef USE_FIPS
+    if(!FIPS_mode_set(global_options.option.fips)) {
+        ERR_load_crypto_strings();
+        sslerror("FIPS_mode_set");
+        return 0;
+    }
+    s_log(LOG_NOTICE, "FIPS mode %s",
+        global_options.option.fips ? "enabled" : "disabled");
+#endif /* USE_FIPS */
+    if(global_options.compression!=COMP_NONE && !init_compression())
+        return 0;
     if(!init_prng())
-        s_log(LOG_DEBUG, "PRNG seeded successfully");
+        return 0;
+    s_log(LOG_DEBUG, "PRNG seeded successfully");
+    return 1; /* SUCCESS */
 }
 
-static void init_compression(void) {
+static int init_compression(void) {
     int id=0;
     COMP_METHOD *cm=NULL;
     char *name="unknown";
 
-    switch(options.compression) {
+    switch(global_options.compression) {
     case COMP_ZLIB:
         id=0xe0;
         cm=COMP_zlib();
@@ -86,17 +98,18 @@ static void init_compression(void) {
         break;
     default:
         s_log(LOG_ERR, "INTERNAL ERROR: Bad compression method");
-        die(1);
+        return 0;
     }
     if(!cm || cm->type==NID_undef) {
         s_log(LOG_ERR, "Failed to initialize %s compression method", name);
-        die(1);
+        return 0;
     }
     if(SSL_COMP_add_compression_method(id, cm)) {
         s_log(LOG_ERR, "Failed to add %s compression method", name);
-        die(1);
+        return 0;
     }
     s_log(LOG_INFO, "Compression enabled using %s method", name);
+    return 1;
 }
 
 static int init_prng(void) {
@@ -108,12 +121,12 @@ static int init_prng(void) {
 
     filename[0]='\0';
 
-    /* If they specify a rand file on the command line we
+    /* if they specify a rand file on the command line we
        assume that they really do want it, so try it first */
-    if(options.rand_file) {
-        totbytes+=add_rand_file(options.rand_file);
+    if(global_options.rand_file) {
+        totbytes+=add_rand_file(global_options.rand_file);
         if(prng_seeded(totbytes))
-            return 0;
+            return 1;
     }
 
     /* try the $RANDFILE or $HOME/.rnd files */
@@ -122,34 +135,34 @@ static int init_prng(void) {
         filename[STRLEN-1]='\0';        /* just in case */
         totbytes+=add_rand_file(filename);
         if(prng_seeded(totbytes))
-            return 0;
+            return 1;
     }
 
 #ifdef RANDOM_FILE
     totbytes+=add_rand_file(RANDOM_FILE);
     if(prng_seeded(totbytes))
-        return 0;
+        return 1;
 #endif
 
 #ifdef USE_WIN32
     RAND_screen();
     if(prng_seeded(totbytes)) {
         s_log(LOG_DEBUG, "Seeded PRNG with RAND_screen");
-        return 0;
+        return 1;
     }
     s_log(LOG_DEBUG, "RAND_screen failed to sufficiently seed PRNG");
 #else
 
 #if SSLEAY_VERSION_NUMBER>=0x0090581fL
-    if(options.egd_sock) {
-        if((bytes=RAND_egd(options.egd_sock))==-1) {
-            s_log(LOG_WARNING, "EGD Socket %s failed", options.egd_sock);
+    if(global_options.egd_sock) {
+        if((bytes=RAND_egd(global_options.egd_sock))==-1) {
+            s_log(LOG_WARNING, "EGD Socket %s failed", global_options.egd_sock);
             bytes=0;
         } else {
             totbytes+=bytes;
             s_log(LOG_DEBUG, "Snagged %d random bytes from EGD Socket %s",
-                bytes, options.egd_sock);
-            return 0; /* OpenSSL always gets what it needs or fails,
+                bytes, global_options.egd_sock);
+            return 1; /* OpenSSL always gets what it needs or fails,
                          so no need to check if seeded sufficiently */
         }
     }
@@ -160,23 +173,22 @@ static int init_prng(void) {
         totbytes+=bytes;
         s_log(LOG_DEBUG, "Snagged %d random bytes from EGD Socket %s",
             bytes, EGD_SOCKET);
-        return 0;
+        return 1;
     }
 #endif /* EGD_SOCKET */
 
 #endif /* OpenSSL-0.9.5a */
 #endif /* USE_WIN32 */
 
-    /* Try the good-old default /dev/urandom, if available  */
+    /* try the good-old default /dev/urandom, if available  */
     totbytes+=add_rand_file("/dev/urandom");
     if(prng_seeded(totbytes))
-        return 0;
+        return 1;
 
-    /* Random file specified during configure */
-    s_log(LOG_WARNING, "PRNG seeded with %d bytes total", totbytes);
-    s_log(LOG_WARNING,
-        "PRNG may not have been seeded with enough random bytes");
-    return -1; /* FAILED */
+    /* random file specified during configure */
+    s_log(LOG_ERR, "PRNG seeded with %d bytes total", totbytes);
+    s_log(LOG_ERR, "PRNG was not seeded with enough random bytes");
+    return 0; /* FAILED */
 }
 
 /* shortcut to determine if sufficient entropy for PRNG is present */
@@ -187,13 +199,13 @@ static int prng_seeded(int bytes) {
         return 1;
     }
 #else
-    if(bytes>=options.random_bytes) {
+    if(bytes>=global_options.random_bytes) {
         s_log(LOG_DEBUG, "Sufficient entropy in PRNG assumed (>= %d)",
-            options.random_bytes);
+            global_options.random_bytes);
         return 1;
     }
 #endif
-    return 0;        /* assume we don't have enough */
+    return 0; /* assume we don't have enough */
 }
 
 static int add_rand_file(char *filename) {
@@ -203,14 +215,14 @@ static int add_rand_file(char *filename) {
 
     if(stat(filename, &sb))
         return 0;
-    if((readbytes=RAND_load_file(filename, options.random_bytes)))
+    if((readbytes=RAND_load_file(filename, global_options.random_bytes)))
         s_log(LOG_DEBUG, "Snagged %d random bytes from %s",
             readbytes, filename);
     else
         s_log(LOG_INFO, "Unable to retrieve any random data from %s",
             filename);
-    /* Write new random data for future seeding if it's a regular file */
-    if(options.option.rand_write && (sb.st_mode & S_IFREG)){
+    /* write new random data for future seeding if it's a regular file */
+    if(global_options.option.rand_write && (sb.st_mode & S_IFREG)){
         writebytes=RAND_write_file(filename);
         if(writebytes==-1)
             s_log(LOG_WARNING, "Failed to write strong random data to %s - "
@@ -229,12 +241,12 @@ static ENGINE *engines[MAX_ENGINES]; /* table of engines */
 static int current_engine=0;
 static int engine_initialized;
 
-void open_engine(const char *name) {
+char *open_engine(const char *name) {
     s_log(LOG_DEBUG, "Enabling support for engine '%s'", name);
     if(!strcasecmp(name, "auto")) {
         ENGINE_register_all_complete();
         s_log(LOG_DEBUG, "Auto engine support enabled");
-        return;
+        return NULL; /* OK */
     }
 
     close_engine(); /* close the previous one (if specified) */
@@ -242,14 +254,14 @@ void open_engine(const char *name) {
     engine_initialized=0;
     if(!engines[current_engine]) {
         sslerror("ENGINE_by_id");
-        die(1);
+        return "Failed to open the engine";
     }
+    return NULL; /* OK */
 }
 
-void ctrl_engine(const char *cmd, const char *arg) {
+char *ctrl_engine(const char *cmd, const char *arg) {
     if(!strcasecmp(cmd, "INIT")) { /* special control command */
-        init_engine();
-        return;
+        return init_engine();
     }
     if(arg)
         s_log(LOG_DEBUG, "Executing engine control command %s:%s", cmd, arg);
@@ -257,11 +269,12 @@ void ctrl_engine(const char *cmd, const char *arg) {
         s_log(LOG_DEBUG, "Executing engine control command %s", cmd);
     if(!ENGINE_ctrl_cmd_string(engines[current_engine], cmd, arg, 0)) {
         sslerror("ENGINE_ctrl_cmd_string");
-        die(1);
+        return "Failed to execute the engine control command";
     }
+    return NULL; /* OK */
 }
 
-void close_engine() {
+void close_engine(void) {
     if(!engines[current_engine])
         return; /* no engine was opened -> nothing to do */
     init_engine();
@@ -274,9 +287,9 @@ void close_engine() {
 #endif
 }
 
-static void init_engine() {
+static char *init_engine(void) {
     if(engine_initialized)
-        return;
+        return NULL; /* OK */
     engine_initialized=1;
     s_log(LOG_DEBUG, "Initializing engine %d", current_engine+1);
     if(!ENGINE_init(engines[current_engine])) {
@@ -284,13 +297,14 @@ static void init_engine() {
             sslerror("ENGINE_init");
         else
             s_log(LOG_ERR, "Engine %d not initialized", current_engine+1);
-        die(1);
+        return "Engine initialization failed";
     }
     if(!ENGINE_set_default(engines[current_engine], ENGINE_METHOD_ALL)) {
         sslerror("ENGINE_set_default");
-        die(1);
+        return "Selecting default engine failed";
     }
     s_log(LOG_DEBUG, "Engine %d initialized", current_engine+1);
+    return NULL; /* OK */
 }
 
 ENGINE *get_engine(int i) {
@@ -301,4 +315,4 @@ ENGINE *get_engine(int i) {
 
 #endif /* HAVE_OSSL_ENGINE_H */
 
-/* End of ssl.c */
+/* end of ssl.c */
