@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2010 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #ifdef USE_OS2
@@ -79,61 +86,66 @@ unsigned long stunnel_thread_id(void) {
     return ready_head ? ready_head->id : 0;
 }
 
-static CONTEXT *new_context(void) {
-    CONTEXT *ctx;
+static CONTEXT *new_context(int stack_size) {
+    CONTEXT *context;
 
     /* allocate and fill the CONTEXT structure */
-    ctx=malloc(sizeof(CONTEXT));
-    if(!ctx) {
+    context=malloc(sizeof(CONTEXT));
+    if(!context) {
         s_log(LOG_ERR, "Unable to allocate CONTEXT structure");
         return NULL;
     }
-    ctx->id=next_id++;
-    ctx->fds=NULL;
-    ctx->ready=0;
-    /* some manuals claim that initialization of ctx structure is required */
-    if(getcontext(&ctx->ctx)<0) {
-        free(ctx);
+    context->stack=malloc(stack_size);
+    if(!context->stack) {
+        s_log(LOG_ERR, "Unable to allocate CONTEXT stack");
+        return NULL;
+    }
+    context->id=next_id++;
+    context->fds=NULL;
+    context->ready=0;
+    /* some manuals claim that initialization of context structure is required */
+    if(getcontext(&context->context)<0) {
+        free(context->stack);
+        free(context);
         ioerror("getcontext");
         return NULL;
     }
-    ctx->ctx.uc_link=NULL; /* it should never happen */
+    context->context.uc_link=NULL; /* it should never happen */
 #if defined(__sgi) || ARGC==2 /* obsolete ss_sp semantics */
-    ctx->ctx.uc_stack.ss_sp=ctx->stack+STACK_SIZE-8;
+    context->context.uc_stack.ss_sp=context->stack+stack_size-8;
 #else
-    ctx->ctx.uc_stack.ss_sp=ctx->stack;
+    context->context.uc_stack.ss_sp=context->stack;
 #endif
-    ctx->ctx.uc_stack.ss_size=STACK_SIZE;
-    ctx->ctx.uc_stack.ss_flags=0;
+    context->context.uc_stack.ss_size=stack_size;
+    context->context.uc_stack.ss_flags=0;
 
     /* attach to the tail of the ready queue */
-    ctx->next=NULL;
+    context->next=NULL;
     if(ready_tail)
-        ready_tail->next=ctx;
-    ready_tail=ctx;
+        ready_tail->next=context;
+    ready_tail=context;
     if(!ready_head)
-        ready_head=ctx;
-    return ctx;
+        ready_head=context;
+    return context;
 }
 
-/* s_log is not initialized here, but we can use log_raw */
 void sthreads_init(void) {
     /* create the first (listening) context and put it in the running queue */
-    if(!new_context()) {
-        log_raw("Unable create the listening context");
-        exit(1);
+    if(!new_context(DEFAULT_STACK_SIZE)) {
+        s_log(LOG_ERR, "Unable create the listening context");
+        die(1);
     }
 }
 
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
-    CONTEXT *ctx;
+int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
+    CONTEXT *context;
 
     s_log(LOG_DEBUG, "Creating a new context");
-    ctx=new_context();
-    if(!ctx)
+    context=new_context(arg->opt->stack_size);
+    if(!context)
         return -1;
-    s_log(LOG_DEBUG, "Context %ld created", ctx->id);
-    makecontext(&ctx->ctx, (void(*)(void))cli, ARGC, arg);
+    s_log(LOG_DEBUG, "Context %ld created", context->id);
+    makecontext(&context->context, (void(*)(void))cli, ARGC, arg);
     return 0;
 }
 
@@ -157,7 +169,7 @@ static void null_handler(int sig) {
     signal(SIGCHLD, null_handler);
 }
 
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
+int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     switch(fork()) {
     case -1:    /* error */
         if(arg)
@@ -170,7 +182,7 @@ int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
             closesocket(ls);
         signal(SIGCHLD, null_handler);
         cli(arg);
-        exit(0);
+        _exit(0);
     default:    /* parent */
         if(arg)
             free(arg);
@@ -186,7 +198,6 @@ int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
 
 static pthread_mutex_t stunnel_cs[CRIT_SECTIONS];
 static pthread_mutex_t lock_cs[CRYPTO_NUM_LOCKS];
-static pthread_attr_t pth_attr;
 
 void enter_critical_section(SECTION_CODE i) {
     pthread_mutex_lock(stunnel_cs+i);
@@ -198,7 +209,7 @@ void leave_critical_section(SECTION_CODE i) {
 
 static void locking_callback(int mode, int type,
 #ifdef HAVE_OPENSSL
-    const /* Callback definition has been changed in openssl 0.9.3 */
+    const /* callback definition has been changed in openssl 0.9.3 */
 #endif
     char *file, int line) {
     if(mode&CRYPTO_LOCK)
@@ -207,22 +218,33 @@ static void locking_callback(int mode, int type,
         pthread_mutex_unlock(lock_cs+type);
 }
 
-void sthreads_init(void) {
-    int i;
+struct CRYPTO_dynlock_value {
+    pthread_mutex_t mutex;
+};
 
-    /* Initialize stunnel critical sections */
-    for(i=0; i<CRIT_SECTIONS; i++)
-        pthread_mutex_init(stunnel_cs+i, NULL);
+static struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
+        int line) {
+    struct CRYPTO_dynlock_value *value;
 
-    /* Initialize OpenSSL locking callback */
-    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
-        pthread_mutex_init(lock_cs+i, NULL);
-    CRYPTO_set_id_callback(stunnel_thread_id);
-    CRYPTO_set_locking_callback(locking_callback);
+    value=malloc(sizeof(struct CRYPTO_dynlock_value));
+    if(!value)
+        return NULL;
+    pthread_mutex_init(&value->mutex, NULL);
+    return value;
+}
 
-    pthread_attr_init(&pth_attr);
-    pthread_attr_setdetachstate(&pth_attr, PTHREAD_CREATE_DETACHED);
-    pthread_attr_setstacksize(&pth_attr, STACK_SIZE);
+static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
+        const char *file, int line) {
+    if(mode&CRYPTO_LOCK)
+        pthread_mutex_lock(&value->mutex);
+    else
+        pthread_mutex_unlock(&value->mutex);
+}
+
+static void dyn_destroy_function(struct CRYPTO_dynlock_value *value,
+        const char *file, int line) {
+    pthread_mutex_destroy(&value->mutex);
+    free(value);
 }
 
 unsigned long stunnel_process_id(void) {
@@ -233,13 +255,38 @@ unsigned long stunnel_thread_id(void) {
     return (unsigned long)pthread_self();
 }
 
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
+void sthreads_init(void) {
+    int i;
+
+    /* initialize stunnel critical sections */
+    for(i=0; i<CRIT_SECTIONS; i++)
+        pthread_mutex_init(stunnel_cs+i, NULL);
+
+    /* initialize OpenSSL locking callback */
+    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
+        pthread_mutex_init(lock_cs+i, NULL);
+    CRYPTO_set_id_callback(stunnel_thread_id);
+    CRYPTO_set_locking_callback(locking_callback);
+
+    /* initialize OpenSSL dynamic locks callbacks */
+    CRYPTO_set_dynlock_create_callback(dyn_create_function);
+    CRYPTO_set_dynlock_lock_callback(dyn_lock_function);
+    CRYPTO_set_dynlock_destroy_callback(dyn_destroy_function);
+}
+
+int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
+    pthread_attr_t pth_attr;
     pthread_t thread;
 #ifdef HAVE_PTHREAD_SIGMASK
     sigset_t newmask, oldmask;
 
-    /* The idea is that only the main thread handles all the signals with
-     * posix threads.  Signals are blocked for any other thread. */
+    /* initialize attributes for creating new threads */
+    pthread_attr_init(&pth_attr);
+    pthread_attr_setdetachstate(&pth_attr, PTHREAD_CREATE_DETACHED);
+    pthread_attr_setstacksize(&pth_attr, arg->opt->stack_size);
+
+    /* the idea is that only the main thread handles all the signals with
+     * posix threads;  signals are blocked for any other thread */
     sigemptyset(&newmask);
     sigaddset(&newmask, SIGCHLD);
     sigaddset(&newmask, SIGTERM);
@@ -279,7 +326,7 @@ void leave_critical_section(SECTION_CODE i) {
 
 static void locking_callback(int mode, int type,
 #ifdef HAVE_OPENSSL
-    const /* Callback definition has been changed in openssl 0.9.3 */
+    const /* callback definition has been changed in openssl 0.9.3 */
 #endif
     char *file, int line) {
     if(mode&CRYPTO_LOCK)
@@ -288,17 +335,33 @@ static void locking_callback(int mode, int type,
         LeaveCriticalSection(lock_cs+type);
 }
 
-void sthreads_init(void) {
-    int i;
+struct CRYPTO_dynlock_value {
+    CRITICAL_SECTION mutex;
+};
 
-    /* Initialize stunnel critical sections */
-    for(i=0; i<CRIT_SECTIONS; i++)
-        InitializeCriticalSection(stunnel_cs+i);
+static struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
+        int line) {
+    struct CRYPTO_dynlock_value *value;
 
-    /* Initialize OpenSSL locking callback */
-    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
-        InitializeCriticalSection(lock_cs+i);
-    CRYPTO_set_locking_callback(locking_callback);
+    value=malloc(sizeof(struct CRYPTO_dynlock_value));
+    if(!value)
+        return NULL;
+    InitializeCriticalSection(&value->mutex);
+    return value;
+}
+
+static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
+        const char *file, int line) {
+    if(mode&CRYPTO_LOCK)
+        EnterCriticalSection(&value->mutex);
+    else
+        LeaveCriticalSection(&value->mutex);
+}
+
+static void dyn_destroy_function(struct CRYPTO_dynlock_value *value,
+        const char *file, int line) {
+    DeleteCriticalSection(&value->mutex);
+    free(value);
 }
 
 unsigned long stunnel_process_id(void) {
@@ -309,9 +372,27 @@ unsigned long stunnel_thread_id(void) {
     return GetCurrentThreadId() & 0x00ffffff;
 }
 
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
+void sthreads_init(void) {
+    int i;
+
+    /* initialize stunnel critical sections */
+    for(i=0; i<CRIT_SECTIONS; i++)
+        InitializeCriticalSection(stunnel_cs+i);
+
+    /* initialize OpenSSL locking callback */
+    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
+        InitializeCriticalSection(lock_cs+i);
+    CRYPTO_set_locking_callback(locking_callback);
+
+    /* initialize OpenSSL dynamic locks callbacks */
+    CRYPTO_set_dynlock_create_callback(dyn_create_function);
+    CRYPTO_set_dynlock_lock_callback(dyn_lock_function);
+    CRYPTO_set_dynlock_destroy_callback(dyn_destroy_function);
+}
+
+int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     s_log(LOG_DEBUG, "Creating a new thread");
-    if(_beginthread((void(*)(void *))cli, STACK_SIZE, arg)==-1) {
+    if(_beginthread((void(*)(void *))cli, arg->opt->stack_size, arg)==-1) {
         ioerror("_beginthread");
         return -1;
     }
@@ -319,8 +400,8 @@ int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
     return 0;
 }
 
-#endif
- 
+#endif /* USE_WIN32 */
+
 #ifdef USE_OS2
 
 void enter_critical_section(SECTION_CODE i) {
@@ -346,9 +427,9 @@ unsigned long stunnel_thread_id(void) {
     return (unsigned long)ppib->pib_ulpid;
 }
 
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
+int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     s_log(LOG_DEBUG, "Creating a new thread");
-    if(_beginthread((void(*)(void *))cli, NULL, STACK_SIZE, arg)==-1) {
+    if(_beginthread((void(*)(void *))cli, NULL, arg->opt->stack_size, arg)==-1) {
         ioerror("_beginthread");
         return -1;
     }
@@ -356,6 +437,8 @@ int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
     return 0;
 }
 
+#endif /* USE_OS2 */
+
 #ifdef _WIN32_WCE
 
 int _beginthread(void (*start_address)(void *),
@@ -375,9 +458,7 @@ void _endthread(void) {
     ExitThread(0);
 }
 
-#endif /* !defined(_WIN32_WCE) */
-
-#endif /* USE_WIN32 */
+#endif /* _WIN32_WCE */
 
 #ifdef DEBUG_STACK_SIZE
 
@@ -424,4 +505,4 @@ void stack_info(int init) { /* 1-initialize, 0-display */
 
 #endif /* DEBUG_STACK_SIZE */
 
-/* End of sthreads.c */
+/* end of sthreads.c */
