@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2010 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -41,8 +41,8 @@
 /**************************************** prototypes */
 
 /* verify initialization */
-static void load_file_lookup(X509_STORE *, char *);
-static void add_dir_lookup(X509_STORE *, char *);
+static int load_file_lookup(X509_STORE *, char *);
+static int add_dir_lookup(X509_STORE *, char *);
 
 /* verify callback */
 static int verify_callback(int, X509_STORE_CTX *);
@@ -55,20 +55,20 @@ static void log_time(const int, const char *, ASN1_TIME *);
 
 /**************************************** verify initialization */
 
-void verify_init(LOCAL_OPTIONS *section) {
+int verify_init(SERVICE_OPTIONS *section) {
     if(section->verify_level<0)
-        return; /* No certificate verification */
+        return 1; /* no certificate verification */
 
     if(section->verify_level>1 && !section->ca_file && !section->ca_dir) {
         s_log(LOG_ERR,
             "Either CApath or CAfile has to be used for authentication");
-        die(1);
+        return 0;
     }
 
     section->revocation_store=X509_STORE_new();
     if(!section->revocation_store) {
         sslerror("X509_STORE_new");
-        die(1);
+        return 0;
     }
 
     if(section->ca_file) {
@@ -77,14 +77,15 @@ void verify_init(LOCAL_OPTIONS *section) {
             s_log(LOG_ERR, "Error loading verify certificates from %s",
                 section->ca_file);
             sslerror("SSL_CTX_load_verify_locations");
-            die(1);
+            return 0;
         }
         /* list of trusted CAs for the client to choose the right cert */
         SSL_CTX_set_client_CA_list(section->ctx,
             SSL_load_client_CA_file(section->ca_file));
         s_log(LOG_DEBUG, "Loaded verify certificates from %s",
             section->ca_file);
-        load_file_lookup(section->revocation_store, section->ca_file);
+        if(!load_file_lookup(section->revocation_store, section->ca_file))
+            return 0;
     }
 
     if(section->ca_dir) {
@@ -93,14 +94,15 @@ void verify_init(LOCAL_OPTIONS *section) {
             s_log(LOG_ERR, "Error setting verify directory to %s",
                 section->ca_dir);
             sslerror("SSL_CTX_load_verify_locations");
-            die(1);
+            return 0;
         }
         s_log(LOG_DEBUG, "Verify directory set to %s", section->ca_dir);
         add_dir_lookup(section->revocation_store, section->ca_dir);
     }
 
     if(section->crl_file)
-        load_file_lookup(section->revocation_store, section->crl_file);
+        if(!load_file_lookup(section->revocation_store, section->crl_file))
+            return 0;
 
     if(section->crl_dir) {
         section->revocation_store->cache=0; /* don't cache CRLs */
@@ -112,38 +114,41 @@ void verify_init(LOCAL_OPTIONS *section) {
 
     if(section->ca_dir && section->verify_use_only_my)
         s_log(LOG_NOTICE, "Peer certificate location %s", section->ca_dir);
+    return 1; /* OK */
 }
 
-static void load_file_lookup(X509_STORE *store, char *name) {
+static int load_file_lookup(X509_STORE *store, char *name) {
     X509_LOOKUP *lookup;
 
     lookup=X509_STORE_add_lookup(store, X509_LOOKUP_file());
     if(!lookup) {
         sslerror("X509_STORE_add_lookup");
-        die(1);
+        return 0;
     }
     if(!X509_LOOKUP_load_file(lookup, name, X509_FILETYPE_PEM)) {
         s_log(LOG_ERR, "Failed to load %s revocation lookup file", name);
         sslerror("X509_LOOKUP_load_file");
-        die(1);
+        return 0;
     }
     s_log(LOG_DEBUG, "Loaded %s revocation lookup file", name);
+    return 1; /* OK */
 }
 
-static void add_dir_lookup(X509_STORE *store, char *name) {
+static int add_dir_lookup(X509_STORE *store, char *name) {
     X509_LOOKUP *lookup;
 
     lookup=X509_STORE_add_lookup(store, X509_LOOKUP_hash_dir());
     if(!lookup) {
         sslerror("X509_STORE_add_lookup");
-        die(1);
+        return 0;
     }
     if(!X509_LOOKUP_add_dir(lookup, name, X509_FILETYPE_PEM)) {
         s_log(LOG_ERR, "Failed to add %s revocation lookup directory", name);
         sslerror("X509_LOOKUP_add_dir");
-        die(1);
+        return 0;
     }
     s_log(LOG_DEBUG, "Added %s revocation lookup directory", name);
+    return 1; /* OK */
 }
 
 /**************************************** verify callback */
@@ -233,7 +238,7 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx,
 
     /* try to retrieve a CRL corresponding to the _subject_ of
      * the current certificate in order to verify it's integrity */
-    memset((char *)&obj, 0, sizeof(obj));
+    memset((char *)&obj, 0, sizeof obj);
     X509_STORE_CTX_init(&store_ctx, c->opt->revocation_store, NULL, NULL);
     rc=X509_STORE_get_by_subject(&store_ctx, X509_LU_CRL, subject, &obj);
     X509_STORE_CTX_cleanup(&store_ctx);
@@ -281,7 +286,7 @@ static int crl_check(CLI *c, X509_STORE_CTX *callback_ctx,
 
     /* try to retrieve a CRL corresponding to the _issuer_ of
      * the current certificate in order to check for revocation */
-    memset((char *)&obj, 0, sizeof(obj));
+    memset((char *)&obj, 0, sizeof obj);
     X509_STORE_CTX_init(&store_ctx, c->opt->revocation_store, NULL, NULL);
     rc=X509_STORE_get_by_subject(&store_ctx, X509_LU_CRL, issuer, &obj);
     X509_STORE_CTX_cleanup(&store_ctx);
@@ -346,16 +351,9 @@ static int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx,
     }
     if(alloc_fd(c->fd))
         goto cleanup;
-    memcpy(&addr, &c->opt->ocsp_addr.addr[0], sizeof(SOCKADDR_UNION));
-    if(connect(c->fd, &addr.sa, addr_len(addr))) {
-        error=get_last_socket_error();
-        if(error!=EINPROGRESS && error!=EWOULDBLOCK) {
-            sockerror("OCSP server connect");
-            goto cleanup;
-        }
-        if(connect_wait(c))
-            goto cleanup;
-    }
+    memcpy(&addr, &c->opt->ocsp_addr.addr[0], sizeof addr);
+    if(connect_blocking(c, &addr, addr_len(addr)))
+        goto cleanup;
     s_log(LOG_DEBUG, "OCSP: server connected");
 
     /* get current certificate ID */
@@ -478,4 +476,4 @@ static void log_time(const int level, const char *txt, ASN1_TIME *t) {
     free(cp);
 }
 
-/* End of verify.c */
+/* end of verify.c */
